import { __awaiter } from "tslib";
/* eslint-disable @typescript-eslint/no-magic-numbers */
/**
 * Temp using this console.log to track versions in devtools by bumping the number for debugging purposes only.
 * If I make a functional change, I bump it...
 * TODO: Remove when this matures.
 */
import { CacheableResponsePlugin } from 'workbox-cacheable-response';
import { clientsClaim } from 'workbox-core';
import { ExpirationPlugin } from 'workbox-expiration';
import { precacheAndRoute } from 'workbox-precaching';
import { registerRoute } from 'workbox-routing';
import { CacheFirst, NetworkFirst, StaleWhileRevalidate } from 'workbox-strategies';
console.log('Hello from sw.js v0.0.1');
clientsClaim();
/**
 * Precache injection point
 */
precacheAndRoute(self.__WB_MANIFEST || []);
/**
 * Cache page navigations (html) with a Network First strategy
 */
registerRoute(({ request }) => request.mode === 'navigate', new NetworkFirst({
    cacheName: 'pages',
    plugins: [
        new CacheableResponsePlugin({
            statuses: [200],
        }),
    ],
}));
/**
 * Caching CSS, JS, and Web Worker requests with a Stale While Revalidate strategy
 */
registerRoute(({ request }) => request.destination === 'style' ||
    request.destination === 'script' ||
    request.destination === 'worker', new StaleWhileRevalidate({
    cacheName: 'assets',
    plugins: [
        new CacheableResponsePlugin({
            statuses: [200],
        }),
    ],
}));
/**
 * Cache images with a Cache First strategy
 */
registerRoute(({ request }) => request.destination === 'image', new CacheFirst({
    cacheName: 'images',
    plugins: [
        new CacheableResponsePlugin({
            statuses: [200],
        }),
        /**
         * Don't cache more than 50 items, and expire them after 30 days
         */
        new ExpirationPlugin({
            maxEntries: 50,
            maxAgeSeconds: 60 * 60 * 24 * 30,
        }),
    ],
}));
/**
 * Message handling
 */
export var ServiceWorkerMessageType;
(function (ServiceWorkerMessageType) {
    ServiceWorkerMessageType["skipWaiting"] = "skipWaiting";
})(ServiceWorkerMessageType || (ServiceWorkerMessageType = {}));
self.addEventListener('message', (event) => {
    if (!event.data) {
        return;
    }
    switch (event.data.type) {
        case ServiceWorkerMessageType.skipWaiting:
            self.skipWaiting();
            break;
        default:
            break;
    }
});
/**
 * Push handling
 */
const urlBase64ToUint8Array = (base64String) => {
    const padding = '='.repeat((4 - (base64String.length % 4)) % 4);
    const base64 = (base64String + padding).replace(/-/g, '+').replace(/_/g, '/');
    const rawData = self.atob(base64);
    const outputArray = new Uint8Array(rawData.length);
    for (let i = 0; i < rawData.length; i += 1) {
        outputArray[i] = rawData.charCodeAt(i);
    }
    return outputArray;
};
/**
 * vapidPublicKey generated using npm run generate-vapid-keys in the project root
 */
const vapidPublicKey = 'BHjc7KuOaU4j6MNOs_Mm90By-mPPUqn8Y94I0yv-aYP03ip2nfq8ti_n1mH01QxW9mWtjY8d2-VdcblQYl1Zq-M';
const pushSubscriptionOptions = {
    applicationServerKey: urlBase64ToUint8Array(vapidPublicKey),
    userVisibleOnly: true,
};
/**
 * TODO: Needs to be configurable app to app, dev vs prod etc
 */
const pushSaveEndpoint = 'http://localhost:6050/push/save';
const savePushSubscription = (pushSubscription) => __awaiter(void 0, void 0, void 0, function* () {
    const response = yield fetch(pushSaveEndpoint, {
        method: 'post',
        headers: {
            'Content-Type': 'application/json',
        },
        body: JSON.stringify(pushSubscription),
    });
    return response.json();
});
self.addEventListener('activate', () => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const pushSubscription = yield self.registration.pushManager.subscribe(pushSubscriptionOptions);
        const response = yield savePushSubscription(pushSubscription);
        console.log(response);
    }
    catch (err) {
        console.error('SW activate error', err);
    }
}));
self.addEventListener('push', (event) => {
    console.log('SW received a push message', event);
    const { data } = event;
    if (!data) {
        console.error('no data in message');
        return;
    }
    const title = data.text() || 'Push Message Title';
    event.waitUntil(self.registration.showNotification(title, {
        tag: 'genesis-global-notification-tag',
        renotify: true,
        body: data.text(),
        vibrate: [500, 100, 500],
    }));
});
self.addEventListener('notificationclose', ({ notification }) => {
    const data = notification.data;
    console.log('SW notification close', data);
});
self.addEventListener('notificationclick', ({ action, notification }) => {
    const data = notification.data;
    console.log('SW notification click', data, action);
    if (action === 'close') {
        notification.close();
        return;
    }
    /**
     * Do something else.
     *
     * self.clients.openWindow('http://www.whatever.com'); // use data
     * notification.close();
     */
});
